@@ -1,5 +1,4 @@
 //go:build linux
-// +build linux
 
 package netavark_test
 
@@ -13,6 +12,7 @@ import (
 	"github.com/containers/common/libnetwork/netavark"
 	"github.com/containers/common/libnetwork/types"
 	"github.com/containers/common/libnetwork/util"
+	"github.com/containers/common/pkg/config"
 	. "github.com/onsi/ginkgo/v2"
 	. "github.com/onsi/gomega"
 	gomegaTypes "github.com/onsi/gomega/types"
@@ -165,7 +165,7 @@ var _ = Describe("Config", func() {
 			network = types.Network{Name: network1.Name}
 			network2, err := libpodNet.NetworkCreate(network, &types.NetworkCreateOptions{IgnoreIfExists: true})
 			Expect(err).To(BeNil())
-			Expect(network2).To(Equal(network1))
+			EqualNetwork(network2, network1)
 		})
 
 		It("create bridge config", func() {
@@ -189,6 +189,32 @@ var _ = Describe("Config", func() {
 			Expect(network1.Internal).To(BeFalse())
 		})
 
+		It("create bridge config with com.docker.network.bridge.name", func() {
+			network := types.Network{
+				Driver: "bridge",
+				Options: map[string]string{
+					"com.docker.network.bridge.name": "foo",
+				},
+			}
+			network1, err := libpodNet.NetworkCreate(network, nil)
+			Expect(err).To(BeNil())
+			Expect(network1.Name).ToNot(BeEmpty())
+			Expect(filepath.Join(networkConfDir, network1.Name+".json")).To(BeARegularFile())
+			Expect(network1.ID).ToNot(BeEmpty())
+			Expect(network1.NetworkInterface).To(Equal("foo"))
+			Expect(network1.Driver).To(Equal("bridge"))
+			Expect(network1.Labels).To(BeEmpty())
+			Expect(network1.Options).To(BeEmpty())
+			Expect(network1.IPAMOptions).ToNot(BeEmpty())
+			Expect(network1.IPAMOptions).To(HaveKeyWithValue("driver", "host-local"))
+			Expect(network1.Subnets).To(HaveLen(1))
+			Expect(network1.Subnets[0].Subnet.String()).To(Equal("10.89.0.0/24"))
+			Expect(network1.Subnets[0].Gateway.String()).To(Equal("10.89.0.1"))
+			Expect(network1.Subnets[0].LeaseRange).To(BeNil())
+			Expect(network1.DNSEnabled).To(BeFalse())
+			Expect(network1.Internal).To(BeFalse())
+		})
+
 		It("create bridge with same name should fail", func() {
 			network := types.Network{
 				Driver:           "bridge",
@@ -253,7 +279,7 @@ var _ = Describe("Config", func() {
 			// reload configs from disk
 			libpodNet, err = getNetworkInterface(networkConfDir)
 			Expect(err).To(BeNil())
-			// check the the networks are identical
+			// check the networks are identical
 			network2, err := libpodNet.NetworkInspect(network1.Name)
 			Expect(err).To(BeNil())
 			EqualNetwork(network2, network1)
@@ -564,6 +590,18 @@ var _ = Describe("Config", func() {
 			Expect(err).To(HaveOccurred())
 		})
 
+		It("create bridge config with invalid com.docker.network.bridge.name", func() {
+			network := types.Network{
+				Driver: "bridge",
+				Options: map[string]string{
+					"com.docker.network.bridge.name": "myname@some",
+				},
+			}
+
+			_, err := libpodNet.NetworkCreate(network, nil)
+			Expect(err).To(HaveOccurred())
+		})
+
 		It("create network with name", func() {
 			name := "myname"
 			network := types.Network{
@@ -646,6 +684,7 @@ var _ = Describe("Config", func() {
 
 		It("update NetworkDNSServers AddDNSServers", func() {
 			libpodNet, err := netavark.NewNetworkInterface(&netavark.InitConfig{
+				Config:           &config.Config{},
 				NetworkConfigDir: networkConfDir,
 				NetworkRunDir:    networkConfDir,
 				NetavarkBinary:   "true",
@@ -666,10 +705,13 @@ var _ = Describe("Config", func() {
 			testNetwork, err := libpodNet.NetworkInspect("test-network")
 			Expect(err).To(BeNil())
 			Expect(testNetwork.NetworkDNSServers).To(Equal([]string{"8.8.8.8", "3.3.3.3", "7.7.7.7"}))
+			err = libpodNet.NetworkUpdate("test-network", types.NetworkUpdateOptions{AddDNSServers: []string{"fake"}})
+			Expect(err).To(HaveOccurred())
 		})
 
 		It("update NetworkDNSServers RemoveDNSServers", func() {
 			libpodNet, err := netavark.NewNetworkInterface(&netavark.InitConfig{
+				Config:           &config.Config{},
 				NetworkConfigDir: networkConfDir,
 				NetworkRunDir:    networkConfDir,
 				NetavarkBinary:   "true",
@@ -690,10 +732,13 @@ var _ = Describe("Config", func() {
 			testNetwork, err := libpodNet.NetworkInspect("test-network")
 			Expect(err).To(BeNil())
 			Expect(testNetwork.NetworkDNSServers).To(Equal([]string{"8.8.8.8"}))
+			err = libpodNet.NetworkUpdate("test-network", types.NetworkUpdateOptions{RemoveDNSServers: []string{"fake"}})
+			Expect(err).To(HaveOccurred())
 		})
 
 		It("update NetworkDNSServers Add and Remove DNSServers", func() {
 			libpodNet, err := netavark.NewNetworkInterface(&netavark.InitConfig{
+				Config:           &config.Config{},
 				NetworkConfigDir: networkConfDir,
 				NetworkRunDir:    networkConfDir,
 				NetavarkBinary:   "true",
@@ -733,7 +778,7 @@ var _ = Describe("Config", func() {
 			}
 			_, err := libpodNet.NetworkCreate(network, nil)
 			Expect(err).To(HaveOccurred())
-			Expect(err.Error()).To(ContainSubstring(`Unable to parse ip a.b.c.d`))
+			Expect(err.Error()).To(ContainSubstring(`unable to parse ip a.b.c.d`))
 		})
 
 		It("create network with NetworDNSServers with DNSEnabled=false", func() {
@@ -743,7 +788,7 @@ var _ = Describe("Config", func() {
 			}
 			_, err := libpodNet.NetworkCreate(network, nil)
 			Expect(err).To(HaveOccurred())
-			Expect(err.Error()).To(ContainSubstring(`Cannot set NetworkDNSServers if DNS is not enabled for the network`))
+			Expect(err.Error()).To(ContainSubstring(`cannot set NetworkDNSServers if DNS is not enabled for the network`))
 		})
 
 		It("create network with labels", func() {
@@ -795,6 +840,43 @@ var _ = Describe("Config", func() {
 			Expect(err.Error()).To(ContainSubstring(`mtu -1 is less than zero`))
 		})
 
+		It("create network with com.docker.network.driver.mtu option", func() {
+			network := types.Network{
+				Options: map[string]string{
+					"com.docker.network.driver.mtu": "1500",
+				},
+			}
+			network1, err := libpodNet.NetworkCreate(network, nil)
+			Expect(err).To(BeNil())
+			Expect(network1.Driver).To(Equal("bridge"))
+			Expect(network1.Options).ToNot(BeNil())
+			path := filepath.Join(networkConfDir, network1.Name+".json")
+			Expect(path).To(BeARegularFile())
+			grepInFile(path, `"mtu": "1500"`)
+			Expect(network1.Options).To(HaveKeyWithValue("mtu", "1500"))
+			Expect(network1.Options).ToNot(HaveKeyWithValue("com.docker.network.driver.mtu", "1500"))
+		})
+
+		It("create network with invalid com.docker.network.driver.mtu option", func() {
+			network := types.Network{
+				Options: map[string]string{
+					"com.docker.network.driver.mtu": "abc",
+				},
+			}
+			_, err := libpodNet.NetworkCreate(network, nil)
+			Expect(err).To(HaveOccurred())
+			Expect(err.Error()).To(ContainSubstring(`parsing "abc": invalid syntax`))
+
+			network = types.Network{
+				Options: map[string]string{
+					"com.docker.network.driver.mtu": "-1",
+				},
+			}
+			_, err = libpodNet.NetworkCreate(network, nil)
+			Expect(err).To(HaveOccurred())
+			Expect(err.Error()).To(ContainSubstring(`mtu -1 is less than zero`))
+		})
+
 		It("create network with vlan option", func() {
 			network := types.Network{
 				Options: map[string]string{
@@ -831,6 +913,22 @@ var _ = Describe("Config", func() {
 			Expect(err.Error()).To(ContainSubstring(`vlan ID -1 must be between 0 and 4094`))
 		})
 
+		It("create network with vrf option", func() {
+			network := types.Network{
+				Options: map[string]string{
+					types.VRFOption: "test",
+				},
+			}
+			network1, err := libpodNet.NetworkCreate(network, nil)
+			Expect(err).To(BeNil())
+			Expect(network1.Driver).To(Equal("bridge"))
+			Expect(network1.Options).ToNot(BeNil())
+			path := filepath.Join(networkConfDir, network1.Name+".json")
+			Expect(path).To(BeARegularFile())
+			grepInFile(path, `"vrf": "test"`)
+			Expect(network1.Options).To(HaveKeyWithValue("vrf", "test"))
+		})
+
 		It("network create unsupported option", func() {
 			network := types.Network{Options: map[string]string{
 				"someopt": "",
@@ -846,7 +944,7 @@ var _ = Describe("Config", func() {
 			}
 			_, err := libpodNet.NetworkCreate(network, nil)
 			Expect(err).To(HaveOccurred())
-			Expect(err.Error()).To(ContainSubstring("unsupported driver someDriver"))
+			Expect(err.Error()).To(ContainSubstring(`failed to find driver or plugin "someDriver"`))
 		})
 
 		It("network create internal and dns", func() {
@@ -917,9 +1015,32 @@ var _ = Describe("Config", func() {
 
 		It("create macvlan config without subnet", func() {
 			network := types.Network{Driver: "macvlan"}
+			network1, err := libpodNet.NetworkCreate(network, nil)
+			Expect(err).ToNot(HaveOccurred())
+			Expect(network1.IPAMOptions[types.Driver]).To(Equal(types.DHCPIPAMDriver))
+		})
+
+		It("create two macvlan networks without name", func() {
+			network := types.Network{Driver: "macvlan"}
+			network1, err := libpodNet.NetworkCreate(network, nil)
+			Expect(err).ToNot(HaveOccurred())
+			Expect(network1.IPAMOptions[types.Driver]).To(Equal(types.DHCPIPAMDriver))
+			Expect(network1.Name).To(Equal("podman1"))
+
+			network2, err := libpodNet.NetworkCreate(network, nil)
+			Expect(err).ToNot(HaveOccurred())
+			Expect(network2.IPAMOptions[types.Driver]).To(Equal(types.DHCPIPAMDriver))
+			Expect(network2.Name).To(Equal("podman2"), "second name should be different then first")
+		})
+
+		It("create macvlan config without subnet and host-local", func() {
+			network := types.Network{
+				Driver:      "macvlan",
+				IPAMOptions: map[string]string{types.Driver: types.HostLocalIPAMDriver},
+			}
 			_, err := libpodNet.NetworkCreate(network, nil)
 			Expect(err).To(HaveOccurred())
-			Expect(err.Error()).To(Equal("macvlan driver needs at least one subnet specified, DHCP is not yet supported with netavark"))
+			Expect(err.Error()).To(ContainSubstring("macvlan driver needs at least one subnet specified when the host-local ipam driver is set"))
 		})
 
 		It("create macvlan config with internal", func() {
@@ -1013,6 +1134,68 @@ var _ = Describe("Config", func() {
 			Expect(network1.IPAMOptions).To(HaveKeyWithValue("driver", "host-local"))
 		})
 
+		It("create macvlan config with bclim", func() {
+			subnet := "10.1.0.0/24"
+			n, _ := types.ParseCIDR(subnet)
+			network := types.Network{
+				Driver: "macvlan",
+				Subnets: []types.Subnet{
+					{Subnet: n},
+				},
+				Options: map[string]string{
+					types.BclimOption: "-1",
+				},
+			}
+			network1, err := libpodNet.NetworkCreate(network, nil)
+			Expect(err).ToNot(HaveOccurred())
+			Expect(network1.Name).ToNot(BeEmpty())
+			Expect(network1.Options).To(HaveKeyWithValue("bclim", "-1"))
+
+			network = types.Network{
+				Driver: "macvlan",
+				Subnets: []types.Subnet{
+					{Subnet: n},
+				},
+				Options: map[string]string{
+					types.BclimOption: "1000",
+				},
+			}
+			network1, err = libpodNet.NetworkCreate(network, nil)
+			Expect(err).ToNot(HaveOccurred())
+			Expect(network1.Name).ToNot(BeEmpty())
+			Expect(network1.Options).To(HaveKeyWithValue("bclim", "1000"))
+
+			network = types.Network{
+				Driver: "macvlan",
+				Subnets: []types.Subnet{
+					{Subnet: n},
+				},
+				Options: map[string]string{
+					types.BclimOption: "abc",
+				},
+			}
+			_, err = libpodNet.NetworkCreate(network, nil)
+			Expect(err).To(HaveOccurred())
+			Expect(err.Error()).To(Equal("failed to parse \"bclim\" option: strconv.ParseInt: parsing \"abc\": invalid syntax"))
+		})
+
+		It("create ipvlan config with bclim should fail", func() {
+			subnet := "10.1.0.0/24"
+			n, _ := types.ParseCIDR(subnet)
+			network := types.Network{
+				Driver: "ipvlan",
+				Subnets: []types.Subnet{
+					{Subnet: n},
+				},
+				Options: map[string]string{
+					types.BclimOption: "-1",
+				},
+			}
+			_, err := libpodNet.NetworkCreate(network, nil)
+			Expect(err).To(HaveOccurred())
+			Expect(err.Error()).To(Equal("unsupported ipvlan network option bclim"))
+		})
+
 		It("create macvlan config with mode", func() {
 			subnet := "10.1.0.0/24"
 			n, _ := types.ParseCIDR(subnet)
@@ -1148,7 +1331,97 @@ var _ = Describe("Config", func() {
 			EqualNetwork(network2, network1)
 		})
 
-		It("create network with isolate option", func() {
+		It("create ipvlan config without subnet", func() {
+			network := types.Network{Driver: "ipvlan"}
+			_, err := libpodNet.NetworkCreate(network, nil)
+			Expect(err).To(HaveOccurred())
+			Expect(err.Error()).To(Equal("ipam driver dhcp is not supported with ipvlan"))
+		})
+
+		It("create ipvlan config without subnet and host-local", func() {
+			network := types.Network{
+				Driver:      "ipvlan",
+				IPAMOptions: map[string]string{types.Driver: types.HostLocalIPAMDriver},
+			}
+			_, err := libpodNet.NetworkCreate(network, nil)
+			Expect(err).To(HaveOccurred())
+			Expect(err.Error()).To(ContainSubstring("ipvlan driver needs at least one subnet specified when the host-local ipam driver is set"))
+		})
+
+		It("create ipvlan config with subnet", func() {
+			subnet := "10.1.0.0/24"
+			n, _ := types.ParseCIDR(subnet)
+			network := types.Network{
+				Driver: "ipvlan",
+				Subnets: []types.Subnet{
+					{Subnet: n},
+				},
+			}
+			network1, err := libpodNet.NetworkCreate(network, nil)
+			Expect(err).To(BeNil())
+			Expect(network1.Name).ToNot(BeEmpty())
+			path := filepath.Join(networkConfDir, network1.Name+".json")
+			Expect(path).To(BeARegularFile())
+			Expect(network1.ID).ToNot(BeEmpty())
+			Expect(network1.Driver).To(Equal("ipvlan"))
+			Expect(network1.NetworkInterface).To(Equal(""))
+			Expect(network1.Labels).To(BeEmpty())
+			Expect(network1.Options).To(BeEmpty())
+			Expect(network1.Subnets).To(HaveLen(1))
+			Expect(network1.Subnets[0].Subnet.String()).To(Equal(subnet))
+			Expect(network1.Subnets[0].Gateway.String()).To(Equal("10.1.0.1"))
+			Expect(network1.Subnets[0].LeaseRange).To(BeNil())
+			Expect(network1.DNSEnabled).To(BeFalse())
+			Expect(network1.Internal).To(BeFalse())
+			Expect(network1.IPAMOptions).To(HaveKeyWithValue("driver", "host-local"))
+		})
+
+		It("create ipvlan config with dhcp driver", func() {
+			network := types.Network{
+				Driver:      "ipvlan",
+				IPAMOptions: map[string]string{types.Driver: types.DHCPIPAMDriver},
+			}
+			_, err := libpodNet.NetworkCreate(network, nil)
+			Expect(err).To(HaveOccurred())
+			Expect(err.Error()).To(Equal("ipam driver dhcp is not supported with ipvlan"))
+		})
+
+		It("create ipvlan config with mode", func() {
+			subnet := "10.1.0.0/24"
+			n, _ := types.ParseCIDR(subnet)
+			network := types.Network{
+				Driver: "ipvlan",
+				Subnets: []types.Subnet{
+					{Subnet: n},
+				},
+				Options: map[string]string{
+					types.ModeOption: "l2",
+				},
+			}
+			network1, err := libpodNet.NetworkCreate(network, nil)
+			Expect(err).To(BeNil())
+			Expect(network1.Name).ToNot(BeEmpty())
+			Expect(network1.Options).To(HaveKeyWithValue("mode", "l2"))
+		})
+
+		It("create ipvlan config with invalid mode", func() {
+			subnet := "10.1.0.0/24"
+			n, _ := types.ParseCIDR(subnet)
+			network := types.Network{
+				Driver: "ipvlan",
+				Subnets: []types.Subnet{
+					{Subnet: n},
+				},
+				Options: map[string]string{
+					types.ModeOption: "abc",
+				},
+			}
+			_, err := libpodNet.NetworkCreate(network, nil)
+			Expect(err).To(HaveOccurred())
+			Expect(err.Error()).To(Equal("unknown ipvlan mode \"abc\""))
+		})
+
+		It("create network with isolate option 'true'", func() {
 			for _, val := range []string{"true", "1"} {
 				network := types.Network{
 					Options: map[string]string{
@@ -1166,6 +1439,22 @@ var _ = Describe("Config", func() {
 			}
 		})
 
+		It("create network with isolate option 'strict'", func() {
+			network := types.Network{
+				Options: map[string]string{
+					types.IsolateOption: "strict",
+				},
+			}
+			network1, err := libpodNet.NetworkCreate(network, nil)
+			Expect(err).To(BeNil())
+			Expect(network1.Driver).To(Equal("bridge"))
+			Expect(network1.Options).ToNot(BeNil())
+			path := filepath.Join(networkConfDir, network1.Name+".json")
+			Expect(path).To(BeARegularFile())
+			grepInFile(path, `"isolate": "strict"`)
+			Expect(network1.Options).To(HaveKeyWithValue("isolate", "strict"))
+		})
+
 		It("create network with invalid isolate option", func() {
 			network := types.Network{
 				Options: map[string]string{
@@ -1177,6 +1466,375 @@ var _ = Describe("Config", func() {
 		})
 	})
 
+	It("create bridge config with static route", func() {
+		dest := "10.1.0.0/24"
+		gw := "10.1.0.1"
+		d, _ := types.ParseCIDR(dest)
+		g := net.ParseIP(gw)
+		network := types.Network{
+			Driver: "bridge",
+			Routes: []types.Route{
+				{Destination: d, Gateway: g},
+			},
+		}
+		network1, err := libpodNet.NetworkCreate(network, nil)
+		Expect(err).To(BeNil())
+		Expect(network1.Name).ToNot(BeEmpty())
+		Expect(network1.Routes).To(HaveLen(1))
+		Expect(network1.Routes[0].Destination.String()).To(Equal(dest))
+		Expect(network1.Routes[0].Gateway.String()).To(Equal(gw))
+	})
+
+	It("create macvlan config with static route", func() {
+		dest := "10.1.0.0/24"
+		gw := "10.1.0.1"
+		d, _ := types.ParseCIDR(dest)
+		g := net.ParseIP(gw)
+		network := types.Network{
+			Driver: "macvlan",
+			Routes: []types.Route{
+				{Destination: d, Gateway: g},
+			},
+		}
+		network1, err := libpodNet.NetworkCreate(network, nil)
+		Expect(err).To(BeNil())
+		Expect(network1.Name).ToNot(BeEmpty())
+		Expect(network1.Routes).To(HaveLen(1))
+		Expect(network1.Routes[0].Destination.String()).To(Equal(dest))
+		Expect(network1.Routes[0].Gateway.String()).To(Equal(gw))
+	})
+
+	It("create ipvlan config with static route", func() {
+		subnet := "10.1.0.0/24"
+		n, _ := types.ParseCIDR(subnet)
+		dest := "10.1.0.0/24"
+		gw := "10.1.0.1"
+		d, _ := types.ParseCIDR(dest)
+		g := net.ParseIP(gw)
+		network := types.Network{
+			Driver: "ipvlan",
+			Subnets: []types.Subnet{
+				{Subnet: n},
+			},
+			Routes: []types.Route{
+				{Destination: d, Gateway: g},
+			},
+		}
+
+		network1, err := libpodNet.NetworkCreate(network, nil)
+		Expect(err).To(BeNil())
+		Expect(network1.Name).ToNot(BeEmpty())
+		Expect(network1.Routes).To(HaveLen(1))
+		Expect(network1.Routes[0].Destination.String()).To(Equal(dest))
+		Expect(network1.Routes[0].Gateway.String()).To(Equal(gw))
+	})
+
+	It("create bridge config with static route (ipv6)", func() {
+		dest := "fd:1234::/64"
+		gw := "fd:4321::1"
+		d, _ := types.ParseCIDR(dest)
+		g := net.ParseIP(gw)
+		network := types.Network{
+			Driver: "bridge",
+			Routes: []types.Route{
+				{Destination: d, Gateway: g},
+			},
+		}
+		network1, err := libpodNet.NetworkCreate(network, nil)
+		Expect(err).To(BeNil())
+		Expect(network1.Name).ToNot(BeEmpty())
+		Expect(network1.Routes).To(HaveLen(1))
+		Expect(network1.Routes[0].Destination.String()).To(Equal(dest))
+		Expect(network1.Routes[0].Gateway.String()).To(Equal(gw))
+	})
+
+	It("create macvlan config with static route (ipv6)", func() {
+		dest := "fd:1234::/64"
+		gw := "fd:4321::1"
+		d, _ := types.ParseCIDR(dest)
+		g := net.ParseIP(gw)
+		network := types.Network{
+			Driver: "macvlan",
+			Routes: []types.Route{
+				{Destination: d, Gateway: g},
+			},
+		}
+		network1, err := libpodNet.NetworkCreate(network, nil)
+		Expect(err).To(BeNil())
+		Expect(network1.Name).ToNot(BeEmpty())
+		Expect(network1.Routes).To(HaveLen(1))
+		Expect(network1.Routes[0].Destination.String()).To(Equal(dest))
+		Expect(network1.Routes[0].Gateway.String()).To(Equal(gw))
+	})
+
+	It("create ipvlan config with static route (ipv6)", func() {
+		subnet := "fd:4321::/64"
+		n, _ := types.ParseCIDR(subnet)
+		dest := "fd:1234::/64"
+		gw := "fd:4321::1"
+		d, _ := types.ParseCIDR(dest)
+		g := net.ParseIP(gw)
+		network := types.Network{
+			Driver: "ipvlan",
+			Subnets: []types.Subnet{
+				{Subnet: n},
+			},
+			Routes: []types.Route{
+				{Destination: d, Gateway: g},
+			},
+		}
+		network1, err := libpodNet.NetworkCreate(network, nil)
+		Expect(err).To(BeNil())
+		Expect(network1.Name).ToNot(BeEmpty())
+		Expect(network1.Routes).To(HaveLen(1))
+		Expect(network1.Routes[0].Destination.String()).To(Equal(dest))
+		Expect(network1.Routes[0].Gateway.String()).To(Equal(gw))
+	})
+
+	It("create bridge config with invalid static route (destination is address)", func() {
+		dest := "10.0.11.10/24"
+		gw := "10.1.0.1"
+		d, _ := types.ParseCIDR(dest)
+		g := net.ParseIP(gw)
+		network := types.Network{
+			Driver: "bridge",
+			Routes: []types.Route{
+				{Destination: d, Gateway: g},
+			},
+		}
+		_, err := libpodNet.NetworkCreate(network, nil)
+		Expect(err).To(HaveOccurred())
+		Expect(err.Error()).To(Equal("route destination invalid"))
+	})
+
+	It("create macvlan config with invalid static route (destination is address)", func() {
+		dest := "10.0.11.10/24"
+		gw := "10.1.0.1"
+		d, _ := types.ParseCIDR(dest)
+		g := net.ParseIP(gw)
+		network := types.Network{
+			Driver: "macvlan",
+			Routes: []types.Route{
+				{Destination: d, Gateway: g},
+			},
+		}
+		_, err := libpodNet.NetworkCreate(network, nil)
+		Expect(err).To(HaveOccurred())
+		Expect(err.Error()).To(Equal("route destination invalid"))
+	})
+
+	It("create ipvlan config with invalid static route (destination is address)", func() {
+		subnet := "10.1.0.0/24"
+		n, _ := types.ParseCIDR(subnet)
+		dest := "10.0.11.10/24"
+		gw := "10.1.0.1"
+		d, _ := types.ParseCIDR(dest)
+		g := net.ParseIP(gw)
+		network := types.Network{
+			Driver: "ipvlan",
+			Subnets: []types.Subnet{
+				{Subnet: n},
+			},
+			Routes: []types.Route{
+				{Destination: d, Gateway: g},
+			},
+		}
+		_, err := libpodNet.NetworkCreate(network, nil)
+		Expect(err).To(HaveOccurred())
+		Expect(err.Error()).To(Equal("route destination invalid"))
+	})
+
+	It("create bridge config with invalid static route (dest = \"foo\")", func() {
+		dest := "foo"
+		gw := "10.1.0.1"
+		d, _ := types.ParseCIDR(dest)
+		g := net.ParseIP(gw)
+		network := types.Network{
+			Driver: "bridge",
+			Routes: []types.Route{
+				{Destination: d, Gateway: g},
+			},
+		}
+		_, err := libpodNet.NetworkCreate(network, nil)
+		Expect(err).To(HaveOccurred())
+		Expect(err.Error()).To(Equal("route destination ip nil"))
+	})
+
+	It("create macvlan config with invalid static route (dest = \"foo\")", func() {
+		dest := "foo"
+		gw := "10.1.0.1"
+		d, _ := types.ParseCIDR(dest)
+		g := net.ParseIP(gw)
+		network := types.Network{
+			Driver: "macvlan",
+			Routes: []types.Route{
+				{Destination: d, Gateway: g},
+			},
+		}
+		_, err := libpodNet.NetworkCreate(network, nil)
+		Expect(err).To(HaveOccurred())
+		Expect(err.Error()).To(Equal("route destination ip nil"))
+	})
+
+	It("create ipvlan config with invalid static route (dest = \"foo\")", func() {
+		subnet := "10.1.0.0/24"
+		n, _ := types.ParseCIDR(subnet)
+		dest := "foo"
+		gw := "10.1.0.1"
+		d, _ := types.ParseCIDR(dest)
+		g := net.ParseIP(gw)
+		network := types.Network{
+			Driver: "ipvlan",
+			Subnets: []types.Subnet{
+				{Subnet: n},
+			},
+			Routes: []types.Route{
+				{Destination: d, Gateway: g},
+			},
+		}
+		_, err := libpodNet.NetworkCreate(network, nil)
+		Expect(err).To(HaveOccurred())
+		Expect(err.Error()).To(Equal("route destination ip nil"))
+	})
+
+	It("create bridge config with invalid static route (gw = \"foo\")", func() {
+		dest := "10.1.0.0/24"
+		gw := "foo"
+		d, _ := types.ParseCIDR(dest)
+		g := net.ParseIP(gw)
+		network := types.Network{
+			Driver: "bridge",
+			Routes: []types.Route{
+				{Destination: d, Gateway: g},
+			},
+		}
+		_, err := libpodNet.NetworkCreate(network, nil)
+		Expect(err).To(HaveOccurred())
+		Expect(err.Error()).To(Equal("route gateway nil"))
+	})
+
+	It("create macvlan config with invalid static route (gw = \"foo\")", func() {
+		dest := "10.1.0.0/24"
+		gw := "foo"
+		d, _ := types.ParseCIDR(dest)
+		g := net.ParseIP(gw)
+		network := types.Network{
+			Driver: "macvlan",
+			Routes: []types.Route{
+				{Destination: d, Gateway: g},
+			},
+		}
+		_, err := libpodNet.NetworkCreate(network, nil)
+		Expect(err).To(HaveOccurred())
+		Expect(err.Error()).To(Equal("route gateway nil"))
+	})
+
+	It("create ipvlan config with invalid static route (gw = \"foo\")", func() {
+		subnet := "10.1.0.0/24"
+		n, _ := types.ParseCIDR(subnet)
+		dest := "10.1.0.0/24"
+		gw := "foo"
+		d, _ := types.ParseCIDR(dest)
+		g := net.ParseIP(gw)
+		network := types.Network{
+			Driver: "ipvlan",
+			Subnets: []types.Subnet{
+				{Subnet: n},
+			},
+			Routes: []types.Route{
+				{Destination: d, Gateway: g},
+			},
+		}
+		_, err := libpodNet.NetworkCreate(network, nil)
+		Expect(err).To(HaveOccurred())
+		Expect(err.Error()).To(Equal("route gateway nil"))
+	})
+
+	It("create bridge config with no_default_route", func() {
+		network := types.Network{
+			Driver: "bridge",
+			Options: map[string]string{
+				"no_default_route": "1",
+			},
+		}
+		network1, err := libpodNet.NetworkCreate(network, nil)
+		Expect(err).To(BeNil())
+		Expect(network1.Options).To(HaveLen(1))
+		Expect(network1.Options["no_default_route"]).To(Equal("true"))
+	})
+
+	It("create macvlan config with no_default_route", func() {
+		network := types.Network{
+			Driver: "macvlan",
+			Options: map[string]string{
+				"no_default_route": "1",
+			},
+		}
+		network1, err := libpodNet.NetworkCreate(network, nil)
+		Expect(err).To(BeNil())
+		Expect(network1.Options).To(HaveLen(1))
+		Expect(network1.Options["no_default_route"]).To(Equal("true"))
+	})
+
+	It("create ipvlan config with no_default_route", func() {
+		subnet := "10.1.0.0/24"
+		n, _ := types.ParseCIDR(subnet)
+		network := types.Network{
+			Driver: "ipvlan",
+			Subnets: []types.Subnet{
+				{Subnet: n},
+			},
+			Options: map[string]string{
+				"no_default_route": "1",
+			},
+		}
+		network1, err := libpodNet.NetworkCreate(network, nil)
+		Expect(err).To(BeNil())
+		Expect(network1.Options).To(HaveLen(1))
+		Expect(network1.Options["no_default_route"]).To(Equal("true"))
+	})
+
+	It("create bridge config with invalid no_default_route", func() {
+		network := types.Network{
+			Driver: "bridge",
+			Options: map[string]string{
+				"no_default_route": "foo",
+			},
+		}
+		_, err := libpodNet.NetworkCreate(network, nil)
+		Expect(err).To(HaveOccurred())
+		Expect(err.Error()).To(ContainSubstring("parsing \"foo\": invalid syntax"))
+	})
+
+	It("create macvlan config with invalid no_default_route", func() {
+		network := types.Network{
+			Driver: "macvlan",
+			Options: map[string]string{
+				"no_default_route": "foo",
+			},
+		}
+		_, err := libpodNet.NetworkCreate(network, nil)
+		Expect(err).To(HaveOccurred())
+		Expect(err.Error()).To(ContainSubstring("parsing \"foo\": invalid syntax"))
+	})
+
+	It("create ipvlan config with invalid no_default_route", func() {
+		subnet := "10.1.0.0/24"
+		n, _ := types.ParseCIDR(subnet)
+		network := types.Network{
+			Driver: "ipvlan",
+			Subnets: []types.Subnet{
+				{Subnet: n},
+			},
+			Options: map[string]string{
+				"no_default_route": "foo",
+			},
+		}
+		_, err := libpodNet.NetworkCreate(network, nil)
+		Expect(err).To(HaveOccurred())
+		Expect(err.Error()).To(ContainSubstring("parsing \"foo\": invalid syntax"))
+	})
+
 	Context("network load valid existing ones", func() {
 		BeforeEach(func() {
 			dir := "testfiles/valid"
@@ -1200,7 +1858,7 @@ var _ = Describe("Config", func() {
 		It("load networks from disk", func() {
 			nets, err := libpodNet.NetworkList()
 			Expect(err).To(BeNil())
-			Expect(nets).To(HaveLen(8))
+			Expect(nets).To(HaveLen(9))
 			// test the we do not show logrus warnings/errors
 			logString := logBuffer.String()
 			Expect(logString).To(BeEmpty())
@@ -1209,12 +1867,12 @@ var _ = Describe("Config", func() {
 		It("change network struct fields should not affect network struct in the backend", func() {
 			nets, err := libpodNet.NetworkList()
 			Expect(err).To(BeNil())
-			Expect(nets).To(HaveLen(8))
+			Expect(nets).To(HaveLen(9))
 
 			nets[0].Name = "myname"
 			nets, err = libpodNet.NetworkList()
 			Expect(err).To(BeNil())
-			Expect(nets).To(HaveLen(8))
+			Expect(nets).To(HaveLen(9))
 			Expect(nets).ToNot(ContainElement(HaveNetworkName("myname")))
 
 			network, err := libpodNet.NetworkInspect("bridge")
@@ -1323,6 +1981,18 @@ var _ = Describe("Config", func() {
 			))
 		})
 
+		It("bridge network with vrf", func() {
+			network, err := libpodNet.NetworkInspect("vrf")
+			Expect(err).To(BeNil())
+			Expect(network.Name).To(Equal("vrf"))
+			Expect(network.ID).To(HaveLen(64))
+			Expect(network.NetworkInterface).To(Equal("podman16"))
+			Expect(network.Driver).To(Equal("bridge"))
+			Expect(network.Subnets).To(HaveLen(1))
+			Expect(network.Options).To(HaveLen(1))
+			Expect(network.Options).To(HaveKeyWithValue("vrf", "test-vrf"))
+		})
+
 		It("network list with filters (name)", func() {
 			filters := map[string][]string{
 				"name": {"internal", "bridge"},
@@ -1362,9 +2032,9 @@ var _ = Describe("Config", func() {
 			Expect(networks).To(ConsistOf(HaveNetworkName("internal"), HaveNetworkName("bridge")))
 		})
 
-		It("network list with filters (id)", func() {
+		It("network list with filters (id with regex)", func() {
 			filters := map[string][]string{
-				"id": {"3bed2cb3a3acf7b6a8ef408420cc682d5520e26976d354254f528c965612054f", "17f29b073143d8cd97b5bbe492bdeffec1c5fee55cc1fe2112c8b9335f8b6121"},
+				"id": {"3bed2cb3a3acf7b6a8ef40.*", "17f29b073143d8cd97b5bbe492bdeffec1c5fee55cc1fe2112c8b9335f8b6121"},
 			}
 			filterFuncs, err := util.GenerateNetworkFilters(filters)
 			Expect(err).To(BeNil())
@@ -1397,10 +2067,11 @@ var _ = Describe("Config", func() {
 
 			networks, err := libpodNet.NetworkList(filterFuncs...)
 			Expect(err).To(BeNil())
-			Expect(networks).To(HaveLen(8))
+			Expect(networks).To(HaveLen(9))
 			Expect(networks).To(ConsistOf(HaveNetworkName("internal"), HaveNetworkName("bridge"),
 				HaveNetworkName("mtu"), HaveNetworkName("vlan"), HaveNetworkName("podman"),
-				HaveNetworkName("label"), HaveNetworkName("dualstack"), HaveNetworkName("metric")))
+				HaveNetworkName("label"), HaveNetworkName("dualstack"), HaveNetworkName("metric"),
+				HaveNetworkName("vrf")))
 		})
 
 		It("network list with filters (label)", func() {
@@ -1490,7 +2161,7 @@ var _ = Describe("Config", func() {
 			logString := logBuffer.String()
 			Expect(logString).To(ContainSubstring("Error reading network config file \\\"%s/broken.json\\\": unexpected EOF", networkConfDir))
 			Expect(logString).To(ContainSubstring("Network config \\\"%s/invalid name.json\\\" has invalid name: \\\"invalid name\\\", skipping: names must match [a-zA-Z0-9][a-zA-Z0-9_.-]*: invalid argument", networkConfDir))
-			Expect(logString).To(ContainSubstring("Network config name \\\"name_miss\\\" does not match file name \\\"name_missmatch.json\\\", skipping"))
+			Expect(logString).To(ContainSubstring("Network config name \\\"name_miss\\\" does not match file name \\\"name_mismatch.json\\\", skipping"))
 			Expect(logString).To(ContainSubstring("Network config \\\"%s/wrongID.json\\\" could not be parsed, skipping: invalid network ID \\\"someID\\\"", networkConfDir))
 			Expect(logString).To(ContainSubstring("Network config \\\"%s/invalid_gateway.json\\\" could not be parsed, skipping: gateway 10.89.100.1 not in subnet 10.89.9.0/24", networkConfDir))
 		})
