@@ -2,8 +2,9 @@ package config
 
 import (
 	"bytes"
-	"io"
+	"fmt"
 	"os"
+	"runtime"
 	"sort"
 	"strings"
 
@@ -16,8 +17,6 @@ import (
 )
 
 var _ = Describe("Config", func() {
-	BeforeEach(beforeEach)
-
 	Describe("ValidateConfig", func() {
 		It("should succeed with default config", func() {
 			// Given
@@ -28,73 +27,99 @@ var _ = Describe("Config", func() {
 			gomega.Expect(err).To(gomega.BeNil())
 			gomega.Expect(defaultConfig.Containers.ApparmorProfile).To(gomega.Equal(apparmor.Profile))
 			gomega.Expect(defaultConfig.Containers.BaseHostsFile).To(gomega.Equal(""))
+			gomega.Expect(defaultConfig.Containers.InterfaceName).To(gomega.Equal(""))
 			gomega.Expect(defaultConfig.Containers.PidsLimit).To(gomega.BeEquivalentTo(2048))
+			gomega.Expect(defaultConfig.Containers.Privileged).To(gomega.BeFalse())
 			gomega.Expect(defaultConfig.Containers.ReadOnly).To(gomega.BeFalse())
 			gomega.Expect(defaultConfig.Engine.ServiceTimeout).To(gomega.BeEquivalentTo(5))
+			gomega.Expect(defaultConfig.Engine.CompressionFormat).To(gomega.BeEquivalentTo("gzip"))
+			gomega.Expect(defaultConfig.Engine.CompressionLevel).To(gomega.BeNil())
 			gomega.Expect(defaultConfig.NetNS()).To(gomega.BeEquivalentTo("private"))
 			gomega.Expect(defaultConfig.IPCNS()).To(gomega.BeEquivalentTo("shareable"))
 			gomega.Expect(defaultConfig.Engine.InfraImage).To(gomega.BeEquivalentTo(""))
-			gomega.Expect(defaultConfig.Engine.ImageVolumeMode).To(gomega.BeEquivalentTo("bind"))
+			gomega.Expect(defaultConfig.Engine.ImageVolumeMode).To(gomega.BeEquivalentTo("anonymous"))
 			gomega.Expect(defaultConfig.Engine.SSHConfig).To(gomega.ContainSubstring("/.ssh/config"))
 			gomega.Expect(defaultConfig.Engine.EventsContainerCreateInspectData).To(gomega.BeFalse())
+			gomega.Expect(defaultConfig.Engine.DBBackend).To(gomega.Equal(""))
+			gomega.Expect(defaultConfig.Engine.PodmanshTimeout).To(gomega.BeEquivalentTo(30))
+			gomega.Expect(defaultConfig.Engine.AddCompression.Get()).To(gomega.HaveLen(0))
+
 			path, err := defaultConfig.ImageCopyTmpDir()
 			gomega.Expect(err).To(gomega.BeNil())
 			gomega.Expect(path).To(gomega.BeEquivalentTo("/var/tmp"))
+			gomega.Expect(defaultConfig.Engine.Retry).To(gomega.BeEquivalentTo(3))
+			gomega.Expect(defaultConfig.Engine.RetryDelay).To(gomega.Equal(""))
 		})
 
 		It("should succeed with devices", func() {
+			defConf, err := defaultConfig()
+			gomega.Expect(err).To(gomega.BeNil())
+			gomega.Expect(defConf).NotTo(gomega.BeNil())
+
 			// Given
-			sut.Containers.Devices = []string{
+			defConf.Containers.Devices.Set([]string{
 				"/dev/null:/dev/null:rw",
 				"/dev/sdc/",
 				"/dev/sdc:/dev/xvdc",
 				"/dev/sdc:rm",
-			}
+			})
 
 			// When
-			err := sut.Containers.Validate()
+			err = defConf.Containers.Validate()
 
 			// Then
 			gomega.Expect(err).To(gomega.BeNil())
 		})
 
 		It("should fail wrong max log size", func() {
+			defConf, err := defaultConfig()
+			gomega.Expect(err).To(gomega.BeNil())
+			gomega.Expect(defConf).NotTo(gomega.BeNil())
+
 			// Given
-			sut.Containers.LogSizeMax = 1
+			defConf.Containers.LogSizeMax = 1
 
 			// When
-			err := sut.Validate()
+			err = defConf.Validate()
 
 			// Then
 			gomega.Expect(err).NotTo(gomega.BeNil())
 		})
 
 		It("should succeed with valid shm size", func() {
+			defConf, err := defaultConfig()
+			gomega.Expect(err).To(gomega.BeNil())
+			gomega.Expect(defConf).NotTo(gomega.BeNil())
+
 			// Given
-			sut.Containers.ShmSize = "1024"
+			defConf.Containers.ShmSize = "1024"
 
 			// When
-			err := sut.Validate()
+			err = defConf.Validate()
 
 			// Then
 			gomega.Expect(err).To(gomega.BeNil())
 
 			// Given
-			sut.Containers.ShmSize = "64m"
+			defConf.Containers.ShmSize = "64m"
 
 			// When
-			err = sut.Validate()
+			err = defConf.Validate()
 
 			// Then
 			gomega.Expect(err).To(gomega.BeNil())
 		})
 
 		It("should fail wrong shm size", func() {
+			defConf, err := defaultConfig()
+			gomega.Expect(err).To(gomega.BeNil())
+			gomega.Expect(defConf).NotTo(gomega.BeNil())
+
 			// Given
-			sut.Containers.ShmSize = "-2"
+			defConf.Containers.ShmSize = "-2"
 
 			// When
-			err := sut.Validate()
+			err = defConf.Validate()
 
 			// Then
 			gomega.Expect(err).NotTo(gomega.BeNil())
@@ -104,14 +129,19 @@ var _ = Describe("Config", func() {
 			defaultConfig, _ := NewConfig("")
 			// EnableLabeling should match whether or not SELinux is enabled on the host
 			gomega.Expect(defaultConfig.Containers.EnableLabeling).To(gomega.Equal(selinux.GetEnabled()))
+			gomega.Expect(defaultConfig.Containers.EnableLabeledUsers).To(gomega.BeFalse())
 		})
 	})
 
 	Describe("ValidateNetworkConfig", func() {
 		It("should succeed with default config", func() {
+			defConf, err := defaultConfig()
+			gomega.Expect(err).To(gomega.BeNil())
+			gomega.Expect(defConf).NotTo(gomega.BeNil())
+
 			// Given
 			// When
-			err := sut.Network.Validate()
+			err = defConf.Network.Validate()
 
 			// Then
 			gomega.Expect(err).To(gomega.BeNil())
@@ -151,11 +181,11 @@ image_copy_tmp_dir="storage"`
 		It("should succeed with default config", func() {
 			// Given
 			// When
-			defaultConfig, _ := DefaultConfig()
+			defaultConfig, _ := defaultConfig()
 			// prior to reading local config, shows hard coded defaults
 			gomega.Expect(defaultConfig.Containers.HTTPProxy).To(gomega.Equal(true))
 
-			err := readConfigFromFile("testdata/containers_default.conf", defaultConfig)
+			err := readConfigFromFile("testdata/containers_default.conf", defaultConfig, false)
 
 			crunWasm := "crun-wasm"
 			PlatformToOCIRuntimeMap := map[string]string{
@@ -191,6 +221,14 @@ image_copy_tmp_dir="storage"`
 					"/usr/bin/crun",
 					"/usr/local/bin/crun",
 				},
+				"crun-vm": {
+					"/usr/bin/crun-vm",
+					"/usr/local/bin/crun-vm",
+					"/usr/local/sbin/crun-vm",
+					"/sbin/crun-vm",
+					"/bin/crun-vm",
+					"/run/current-system/sw/bin/crun-vm",
+				},
 				"crun-wasm": {
 					"/usr/bin/crun-wasm",
 					"/usr/sbin/crun-wasm",
@@ -231,7 +269,11 @@ image_copy_tmp_dir="storage"`
 
 			envs := []string{
 				"PATH=/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin",
-				"TERM=xterm",
+			}
+
+			mounts := []string{
+				"type=glob,source=/tmp/test2*,ro=true",
+				"type=bind,source=/etc/services,destination=/etc/services,ro",
 			}
 
 			volumes := []string{
@@ -249,18 +291,21 @@ image_copy_tmp_dir="storage"`
 			// Then
 			gomega.Expect(err).To(gomega.BeNil())
 			gomega.Expect(defaultConfig.Engine.CgroupManager).To(gomega.Equal("systemd"))
-			gomega.Expect(defaultConfig.Containers.Env).To(gomega.BeEquivalentTo(envs))
+			gomega.Expect(defaultConfig.Containers.Env.Get()).To(gomega.BeEquivalentTo(envs))
+			gomega.Expect(defaultConfig.Containers.Mounts.Get()).To(gomega.BeEquivalentTo(mounts))
 			gomega.Expect(defaultConfig.Containers.PidsLimit).To(gomega.BeEquivalentTo(2048))
-			gomega.Expect(defaultConfig.Network.CNIPluginDirs).To(gomega.Equal(pluginDirs))
+			gomega.Expect(defaultConfig.Network.CNIPluginDirs.Get()).To(gomega.Equal(pluginDirs))
+			gomega.Expect(defaultConfig.Network.NetavarkPluginDirs.Get()).To(gomega.Equal([]string{"/usr/netavark"}))
 			gomega.Expect(defaultConfig.Engine.NumLocks).To(gomega.BeEquivalentTo(2048))
 			gomega.Expect(defaultConfig.Engine.OCIRuntimes).To(gomega.Equal(OCIRuntimeMap))
 			gomega.Expect(defaultConfig.Engine.PlatformToOCIRuntime).To(gomega.Equal(PlatformToOCIRuntimeMap))
 			gomega.Expect(defaultConfig.Containers.HTTPProxy).To(gomega.Equal(false))
-			gomega.Expect(defaultConfig.Engine.NetworkCmdOptions).To(gomega.BeNil())
-			gomega.Expect(defaultConfig.Engine.HelperBinariesDir).To(gomega.Equal(helperDirs))
+			gomega.Expect(defaultConfig.Engine.NetworkCmdOptions.Get()).To(gomega.HaveLen(0))
+			gomega.Expect(defaultConfig.Engine.HelperBinariesDir.Get()).To(gomega.Equal(helperDirs))
 			gomega.Expect(defaultConfig.Engine.ServiceTimeout).To(gomega.BeEquivalentTo(300))
 			gomega.Expect(defaultConfig.Engine.InfraImage).To(gomega.BeEquivalentTo("k8s.gcr.io/pause:3.4.1"))
-			gomega.Expect(defaultConfig.Machine.Volumes).To(gomega.BeEquivalentTo(volumes))
+			gomega.Expect(defaultConfig.Machine.Volumes.Get()).To(gomega.BeEquivalentTo(volumes))
+			gomega.Expect(defaultConfig.Engine.PodmanshTimeout).To(gomega.BeEquivalentTo(300))
 			newV, err := defaultConfig.MachineVolumes()
 			if newVolumes[0] == ":" {
 				// $HOME is not set
@@ -269,12 +314,13 @@ image_copy_tmp_dir="storage"`
 				gomega.Expect(err).To(gomega.BeNil())
 				gomega.Expect(newV).To(gomega.BeEquivalentTo(newVolumes))
 			}
+			gomega.Expect(defaultConfig.Engine.Retry).To(gomega.BeEquivalentTo(5))
+			gomega.Expect(defaultConfig.Engine.RetryDelay).To(gomega.Equal("10s"))
 		})
 
 		It("test GetDefaultEnvEx", func() {
 			envs := []string{
 				"PATH=/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin",
-				"TERM=xterm",
 			}
 			httpEnvs := append([]string{"HTTP_PROXY=1.2.3.4"}, envs...)
 			oldProxy, proxyEnvSet := os.LookupEnv("HTTP_PROXY")
@@ -282,7 +328,7 @@ image_copy_tmp_dir="storage"`
 			oldFoo, fooEnvSet := os.LookupEnv("foo")
 			os.Setenv("foo", "bar")
 
-			defaultConfig, _ := DefaultConfig()
+			defaultConfig, _ := defaultConfig()
 			gomega.Expect(defaultConfig.GetDefaultEnvEx(false, false)).To(gomega.BeEquivalentTo(envs))
 			gomega.Expect(defaultConfig.GetDefaultEnvEx(false, true)).To(gomega.BeEquivalentTo(httpEnvs))
 			gomega.Expect(strings.Join(defaultConfig.GetDefaultEnvEx(true, true), ",")).To(gomega.ContainSubstring("HTTP_PROXY"))
@@ -304,7 +350,7 @@ image_copy_tmp_dir="storage"`
 			// Given
 			// When
 			conf := Config{}
-			err := readConfigFromFile("testdata/containers_comment.conf", &conf)
+			err := readConfigFromFile("testdata/containers_comment.conf", &conf, false)
 
 			// Then
 			gomega.Expect(err).To(gomega.BeNil())
@@ -314,7 +360,7 @@ image_copy_tmp_dir="storage"`
 			// Given
 			// When
 			conf := Config{}
-			err := readConfigFromFile("/invalid/file", &conf)
+			err := readConfigFromFile("/invalid/file", &conf, false)
 
 			// Then
 			gomega.Expect(err).NotTo(gomega.BeNil())
@@ -324,7 +370,7 @@ image_copy_tmp_dir="storage"`
 			// Given
 			// When
 			conf := Config{}
-			err := readConfigFromFile("config.go", &conf)
+			err := readConfigFromFile("config.go", &conf, false)
 
 			// Then
 			gomega.Expect(err).NotTo(gomega.BeNil())
@@ -356,31 +402,50 @@ image_copy_tmp_dir="storage"`
 				},
 			}
 
+			defCaps := []string{
+				"CAP_CHOWN",
+				"CAP_DAC_OVERRIDE",
+				"CAP_FOWNER",
+				"CAP_FSETID",
+				"CAP_KILL",
+				"CAP_NET_BIND_SERVICE",
+				"CAP_SETFCAP",
+				"CAP_SETGID",
+				"CAP_SETPCAP",
+				"CAP_SETUID",
+				"CAP_SYS_CHROOT",
+			}
+
 			envs := []string{
 				"PATH=/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin",
-				"TERM=xterm",
 			}
 
 			// Given we do
-			oldContainersConf, envSet := os.LookupEnv("CONTAINERS_CONF")
-			os.Setenv("CONTAINERS_CONF", "/dev/null")
+			oldContainersConf, envSet := os.LookupEnv(containersConfEnv)
+			os.Setenv(containersConfEnv, "/dev/null")
 			// When
 			config, err := NewConfig("")
 			// Undo that
 			if envSet {
-				os.Setenv("CONTAINERS_CONF", oldContainersConf)
+				os.Setenv(containersConfEnv, oldContainersConf)
 			} else {
-				os.Unsetenv("CONTAINERS_CONF")
+				os.Unsetenv(containersConfEnv)
 			}
 			// Then
 			gomega.Expect(err).To(gomega.BeNil())
 			gomega.Expect(config.Containers.ApparmorProfile).To(gomega.Equal(apparmor.Profile))
 			gomega.Expect(config.Containers.PidsLimit).To(gomega.BeEquivalentTo(2048))
-			gomega.Expect(config.Containers.Env).To(gomega.BeEquivalentTo(envs))
+			gomega.Expect(config.Containers.Env.Get()).To(gomega.BeEquivalentTo(envs))
 			gomega.Expect(config.Containers.UserNS).To(gomega.BeEquivalentTo(""))
-			gomega.Expect(config.Network.CNIPluginDirs).To(gomega.Equal(DefaultCNIPluginDirs))
+			gomega.Expect(config.Network.CNIPluginDirs.Get()).To(gomega.Equal(DefaultCNIPluginDirs))
+			gomega.Expect(config.Network.NetavarkPluginDirs.Get()).To(gomega.Equal(DefaultNetavarkPluginDirs))
 			gomega.Expect(config.Engine.NumLocks).To(gomega.BeEquivalentTo(2048))
 			gomega.Expect(config.Engine.OCIRuntimes["runc"]).To(gomega.Equal(OCIRuntimeMap["runc"]))
+			gomega.Expect(config.Containers.CgroupConf.Get()).To(gomega.HaveLen(0))
+
+			caps, _ := config.Capabilities("", nil, nil)
+			gomega.Expect(caps).Should(gomega.Equal(defCaps))
+
 			if useSystemd() {
 				gomega.Expect(config.Engine.CgroupManager).To(gomega.BeEquivalentTo("systemd"))
 			} else {
@@ -396,6 +461,7 @@ image_copy_tmp_dir="storage"`
 			gomega.Expect(config.Engine.EventsLogFilePath).To(gomega.BeEquivalentTo(""))
 			gomega.Expect(uint64(config.Engine.EventsLogFileMaxSize)).To(gomega.Equal(DefaultEventsLogSizeMax))
 			gomega.Expect(config.Engine.PodExitPolicy).To(gomega.Equal(PodExitPolicyContinue))
+			gomega.Expect(config.Engine.KubeGenerateType).To(gomega.Equal("pod"))
 		})
 
 		It("should success with valid user file path", func() {
@@ -403,6 +469,9 @@ image_copy_tmp_dir="storage"`
 			// When
 			config, err := NewConfig("testdata/containers_default.conf")
 			// Then
+			cgroupConf := []string{
+				"memory.high=1073741824",
+			}
 			gomega.Expect(err).To(gomega.BeNil())
 			gomega.Expect(config.Containers.ApparmorProfile).To(gomega.Equal("container-default"))
 			gomega.Expect(config.Containers.PidsLimit).To(gomega.BeEquivalentTo(2048))
@@ -410,19 +479,24 @@ image_copy_tmp_dir="storage"`
 			gomega.Expect(config.Containers.HostContainersInternalIP).To(gomega.BeEquivalentTo("1.2.3.4"))
 			gomega.Expect(config.Engine.ImageVolumeMode).To(gomega.BeEquivalentTo("tmpfs"))
 			gomega.Expect(config.Engine.SSHConfig).To(gomega.Equal("/foo/bar/.ssh/config"))
+
+			gomega.Expect(config.Engine.DBBackend).To(gomega.Equal(stringSQLite))
+			gomega.Expect(config.Containers.CgroupConf.Get()).To(gomega.Equal(cgroupConf))
+			gomega.Expect(*config.Containers.OOMScoreAdj).To(gomega.Equal(int(750)))
+			gomega.Expect(config.Engine.KubeGenerateType).To(gomega.Equal("pod"))
 		})
 
 		It("contents of passed-in file should override others", func() {
 			// Given we do
-			oldContainersConf, envSet := os.LookupEnv("CONTAINERS_CONF")
-			os.Setenv("CONTAINERS_CONF", "containers.conf")
+			oldContainersConf, envSet := os.LookupEnv(containersConfEnv)
+			os.Setenv(containersConfEnv, "containers.conf")
 			// When
 			config, err := NewConfig("testdata/containers_override.conf")
 			// Undo that
 			if envSet {
-				os.Setenv("CONTAINERS_CONF", oldContainersConf)
+				os.Setenv(containersConfEnv, oldContainersConf)
 			} else {
-				os.Unsetenv("CONTAINERS_CONF")
+				os.Unsetenv(containersConfEnv)
 			}
 
 			crunWasm := "crun-wasm"
@@ -444,10 +518,12 @@ image_copy_tmp_dir="storage"`
 			gomega.Expect(config.Containers.LogDriver).To(gomega.Equal("journald"))
 			gomega.Expect(config.Containers.LogTag).To(gomega.Equal("{{.Name}}|{{.ID}}"))
 			gomega.Expect(config.Containers.LogSizeMax).To(gomega.Equal(int64(100000)))
+			gomega.Expect(config.Containers.Privileged).To(gomega.BeTrue())
 			gomega.Expect(config.Containers.ReadOnly).To(gomega.BeTrue())
 			gomega.Expect(config.Engine.ImageParallelCopies).To(gomega.Equal(uint(10)))
 			gomega.Expect(config.Engine.PlatformToOCIRuntime).To(gomega.Equal(PlatformToOCIRuntimeMap))
 			gomega.Expect(config.Engine.ImageDefaultFormat).To(gomega.Equal("v2s2"))
+			gomega.Expect(config.Engine.CompressionFormat).To(gomega.BeEquivalentTo("zstd:chunked"))
 			gomega.Expect(config.Engine.EventsLogFilePath).To(gomega.BeEquivalentTo("/tmp/events.log"))
 			gomega.Expect(config.Engine.EventsContainerCreateInspectData).To(gomega.BeTrue())
 			path, err := config.ImageCopyTmpDir()
@@ -468,6 +544,9 @@ image_copy_tmp_dir="storage"`
 
 		It("Test Capabilities call", func() {
 			// Given
+			if runtime.GOOS != "linux" {
+				Skip(fmt.Sprintf("capabilities not supported on %s", runtime.GOOS))
+			}
 			// When
 			config, err := NewConfig("")
 			// Then
@@ -476,7 +555,7 @@ image_copy_tmp_dir="storage"`
 			caps, err := config.Capabilities("0", addcaps, dropcaps)
 			gomega.Expect(err).To(gomega.BeNil())
 			sort.Strings(caps)
-			defaultCaps := config.Containers.DefaultCapabilities
+			defaultCaps := config.Containers.DefaultCapabilities.Get()
 			sort.Strings(defaultCaps)
 			gomega.Expect(caps).To(gomega.BeEquivalentTo(defaultCaps))
 
@@ -496,12 +575,12 @@ image_copy_tmp_dir="storage"`
 			sort.Strings(caps)
 			gomega.Expect(caps).ToNot(gomega.BeEquivalentTo([]string{}))
 
-			config.Containers.DefaultCapabilities = []string{
+			config.Containers.DefaultCapabilities.Set([]string{
 				"CAP_AUDIT_WRITE",
 				"CAP_CHOWN",
 				"CAP_DAC_OVERRIDE",
 				"CAP_FOWNER",
-			}
+			})
 
 			expectedCaps := []string{
 				"CAP_AUDIT_WRITE",
@@ -533,26 +612,48 @@ image_copy_tmp_dir="storage"`
 		})
 
 		It("should succeed with default pull_policy", func() {
-			err := sut.Engine.Validate()
+			defConf, err := defaultConfig()
 			gomega.Expect(err).To(gomega.BeNil())
-			gomega.Expect(sut.Engine.PullPolicy).To(gomega.Equal("missing"))
+			gomega.Expect(defConf).NotTo(gomega.BeNil())
 
-			sut.Engine.PullPolicy = DefaultPullPolicy
-			err = sut.Engine.Validate()
+			err = defConf.Engine.Validate()
+			gomega.Expect(err).To(gomega.BeNil())
+			gomega.Expect(defConf.Engine.PullPolicy).To(gomega.Equal("missing"))
+
+			defConf.Engine.PullPolicy = DefaultPullPolicy
+			err = defConf.Engine.Validate()
 			gomega.Expect(err).To(gomega.BeNil())
 		})
 
 		It("should succeed case-insensitive", func() {
-			sut.Engine.PullPolicy = "NeVer"
-			err := sut.Engine.Validate()
+			defConf, err := defaultConfig()
+			gomega.Expect(err).To(gomega.BeNil())
+			gomega.Expect(defConf).NotTo(gomega.BeNil())
+
+			defConf.Engine.PullPolicy = "NeVer"
+			err = defConf.Engine.Validate()
 			gomega.Expect(err).To(gomega.BeNil())
 		})
 
 		It("should fail with invalid pull_policy", func() {
-			sut.Engine.PullPolicy = "invalidPullPolicy"
-			err := sut.Engine.Validate()
+			defConf, err := defaultConfig()
+			gomega.Expect(err).To(gomega.BeNil())
+			gomega.Expect(defConf).NotTo(gomega.BeNil())
+
+			defConf.Engine.PullPolicy = "invalidPullPolicy"
+			err = defConf.Engine.Validate()
 			gomega.Expect(err).ToNot(gomega.BeNil())
 		})
+
+		It("should fail with invalid database_backend", func() {
+			defConf, err := defaultConfig()
+			gomega.Expect(err).To(gomega.BeNil())
+			gomega.Expect(defConf).NotTo(gomega.BeNil())
+
+			defConf.Engine.DBBackend = "blah"
+			err = defConf.Engine.Validate()
+			gomega.Expect(err).To(gomega.HaveOccurred())
+		})
 	})
 
 	Describe("Service Destinations", func() {
@@ -562,180 +663,20 @@ image_copy_tmp_dir="storage"`
 		}{}
 
 		BeforeEach(func() {
-			ConfPath.Value, ConfPath.IsSet = os.LookupEnv("CONTAINERS_CONF")
+			ConfPath.Value, ConfPath.IsSet = os.LookupEnv(containersConfEnv)
 			conf, _ := os.CreateTemp("", "containersconf")
-			os.Setenv("CONTAINERS_CONF", conf.Name())
+			os.Setenv(containersConfEnv, conf.Name())
 		})
 
 		AfterEach(func() {
-			os.Remove(os.Getenv("CONTAINERS_CONF"))
+			os.Remove(os.Getenv(containersConfEnv))
 			if ConfPath.IsSet {
-				os.Setenv("CONTAINERS_CONF", ConfPath.Value)
+				os.Setenv(containersConfEnv, ConfPath.Value)
 			} else {
-				os.Unsetenv("CONTAINERS_CONF")
+				os.Unsetenv(containersConfEnv)
 			}
 		})
 
-		It("succeed to set and read", func() {
-			cfg, err := ReadCustomConfig()
-			gomega.Expect(err).ShouldNot(gomega.HaveOccurred())
-
-			cfg.Engine.ActiveService = "QA"
-			cfg.Engine.ServiceDestinations = map[string]Destination{
-				"QA": {
-					URI:      "https://qa/run/podman/podman.sock",
-					Identity: "/.ssh/id_rsa",
-				},
-			}
-			err = cfg.Write()
-			gomega.Expect(err).ShouldNot(gomega.HaveOccurred())
-
-			// test that we do not write zero values to the file
-			path, err := customConfigFile()
-			gomega.Expect(err).ShouldNot(gomega.HaveOccurred())
-			f, err := os.Open(path)
-			gomega.Expect(err).ShouldNot(gomega.HaveOccurred())
-			data, err := io.ReadAll(f)
-			gomega.Expect(err).ShouldNot(gomega.HaveOccurred())
-			gomega.Expect(string(data)).ShouldNot(gomega.ContainSubstring("cpus"))
-			gomega.Expect(string(data)).ShouldNot(gomega.ContainSubstring("disk_size"))
-			gomega.Expect(string(data)).ShouldNot(gomega.ContainSubstring("memory"))
-
-			cfg, err = ReadCustomConfig()
-			gomega.Expect(err).ShouldNot(gomega.HaveOccurred())
-
-			gomega.Expect(cfg.Engine.ActiveService, "QA")
-			gomega.Expect(cfg.Engine.ServiceDestinations["QA"].URI,
-				"https://qa/run/podman/podman.sock")
-			gomega.Expect(cfg.Engine.ServiceDestinations["QA"].Identity,
-				"/.ssh/id_rsa")
-		})
-
-		It("succeed ActiveDestinations()", func() {
-			cfg, err := ReadCustomConfig()
-			gomega.Expect(err).ShouldNot(gomega.HaveOccurred())
-
-			cfg.Engine.ActiveService = "QA"
-			cfg.Engine.ServiceDestinations = map[string]Destination{
-				"QB": {
-					URI:      "https://qb/run/podman/podman.sock",
-					Identity: "/.ssh/qb_id_rsa",
-				},
-				"QA": {
-					URI:      "https://qa/run/podman/podman.sock",
-					Identity: "/.ssh/id_rsa",
-				},
-			}
-			err = cfg.Write()
-			gomega.Expect(err).ShouldNot(gomega.HaveOccurred())
-
-			cfg, err = ReadCustomConfig()
-			gomega.Expect(err).ShouldNot(gomega.HaveOccurred())
-
-			u, i, m, err := cfg.ActiveDestination()
-			gomega.Expect(err).ShouldNot(gomega.HaveOccurred())
-
-			gomega.Expect(m).To(gomega.Equal(false))
-			gomega.Expect(u).To(gomega.Equal("https://qa/run/podman/podman.sock"))
-			gomega.Expect(i).To(gomega.Equal("/.ssh/id_rsa"))
-		})
-
-		It("succeed ActiveDestinations() CONTAINER_CONNECTION environment", func() {
-			cfg, err := ReadCustomConfig()
-			gomega.Expect(err).ShouldNot(gomega.HaveOccurred())
-
-			cfg.Engine.ActiveService = "QA"
-			cfg.Engine.ServiceDestinations = map[string]Destination{
-				"QA": {
-					URI:      "https://qa/run/podman/podman.sock",
-					Identity: "/.ssh/id_rsa",
-				},
-				"QB": {
-					URI:      "https://qb/run/podman/podman.sock",
-					Identity: "/.ssh/qb_id_rsa",
-				},
-			}
-			err = cfg.Write()
-			gomega.Expect(err).ShouldNot(gomega.HaveOccurred())
-
-			cfg, err = ReadCustomConfig()
-			gomega.Expect(err).ShouldNot(gomega.HaveOccurred())
-
-			// Given we do
-			oldContainerConnection, hostEnvSet := os.LookupEnv("CONTAINER_CONNECTION")
-			os.Setenv("CONTAINER_CONNECTION", "QB")
-
-			u, i, m, err := cfg.ActiveDestination()
-			// Undo that
-			if hostEnvSet {
-				os.Setenv("CONTAINER_CONNECTION", oldContainerConnection)
-			} else {
-				os.Unsetenv("CONTAINER_CONNECTION")
-			}
-			gomega.Expect(err).ShouldNot(gomega.HaveOccurred())
-			gomega.Expect(m).To(gomega.Equal(false))
-
-			gomega.Expect(u).To(gomega.Equal("https://qb/run/podman/podman.sock"))
-			gomega.Expect(i).To(gomega.Equal("/.ssh/qb_id_rsa"))
-		})
-
-		It("succeed ActiveDestinations CONTAINER_HOST ()", func() {
-			cfg, err := ReadCustomConfig()
-			gomega.Expect(err).ShouldNot(gomega.HaveOccurred())
-
-			cfg.Engine.ActiveService = "QA"
-			cfg.Engine.ServiceDestinations = map[string]Destination{
-				"QB": {
-					URI:      "https://qb/run/podman/podman.sock",
-					Identity: "/.ssh/qb_id_rsa",
-				},
-				"QA": {
-					URI:      "https://qa/run/podman/podman.sock",
-					Identity: "/.ssh/id_rsa",
-				},
-			}
-			err = cfg.Write()
-			gomega.Expect(err).ShouldNot(gomega.HaveOccurred())
-
-			cfg, err = ReadCustomConfig()
-			gomega.Expect(err).ShouldNot(gomega.HaveOccurred())
-
-			// Given we do
-			oldContainerHost, hostEnvSet := os.LookupEnv("CONTAINER_HOST")
-			oldContainerSSH, sshEnvSet := os.LookupEnv("CONTAINER_SSHKEY")
-			os.Setenv("CONTAINER_HOST", "foo.bar")
-			os.Setenv("CONTAINER_SSHKEY", "/.ssh/newid_rsa")
-
-			u, i, m, err := cfg.ActiveDestination()
-
-			// Undo that
-			if hostEnvSet {
-				os.Setenv("CONTAINER_HOST", oldContainerHost)
-			} else {
-				os.Unsetenv("CONTAINER_HOST")
-			}
-			// Undo that
-			if sshEnvSet {
-				os.Setenv("CONTAINER_SSHKEY", oldContainerSSH)
-			} else {
-				os.Unsetenv("CONTAINER_SSHKEY")
-			}
-
-			gomega.Expect(err).ShouldNot(gomega.HaveOccurred())
-			gomega.Expect(m).To(gomega.Equal(false))
-
-			gomega.Expect(u).To(gomega.Equal("foo.bar"))
-			gomega.Expect(i).To(gomega.Equal("/.ssh/newid_rsa"))
-		})
-
-		It("fail ActiveDestination() no configuration", func() {
-			cfg, err := ReadCustomConfig()
-			gomega.Expect(err).ShouldNot(gomega.HaveOccurred())
-
-			_, _, _, err = cfg.ActiveDestination()
-			gomega.Expect(err).Should(gomega.HaveOccurred())
-		})
-
 		It("test addConfigs", func() {
 			tmpFilePath := func(dir, prefix string) string {
 				file, err := os.CreateTemp(dir, prefix)
@@ -783,7 +724,7 @@ image_copy_tmp_dir="storage"`
 			content := bytes.NewBufferString("")
 			logrus.SetOutput(content)
 			logrus.SetLevel(logrus.DebugLevel)
-			err := readConfigFromFile("testdata/containers_broken.conf", &conf)
+			err := readConfigFromFile("testdata/containers_broken.conf", &conf, false)
 			gomega.Expect(err).To(gomega.BeNil())
 			gomega.Expect(conf.Containers.NetNS).To(gomega.Equal("bridge"))
 			gomega.Expect(conf.Containers.Umask).To(gomega.Equal("0002"))
@@ -795,7 +736,7 @@ image_copy_tmp_dir="storage"`
 			conf := Config{}
 			content := bytes.NewBufferString("")
 			logrus.SetOutput(content)
-			err := readConfigFromFile("containers.conf", &conf)
+			err := readConfigFromFile("containers.conf", &conf, false)
 			gomega.Expect(err).To(gomega.BeNil())
 			gomega.Expect(content.String()).To(gomega.Equal(""))
 			logrus.SetOutput(os.Stderr)
@@ -806,14 +747,14 @@ image_copy_tmp_dir="storage"`
 		It("test new config from reload", func() {
 			// Default configuration
 			defaultTestFile := "testdata/containers_default.conf"
-			oldEnv, set := os.LookupEnv("CONTAINERS_CONF")
-			os.Setenv("CONTAINERS_CONF", defaultTestFile)
+			oldEnv, set := os.LookupEnv(containersConfEnv)
+			os.Setenv(containersConfEnv, defaultTestFile)
 			cfg, err := Default()
 			gomega.Expect(err).To(gomega.BeNil())
 			if set {
-				os.Setenv("CONTAINERS_CONF", oldEnv)
+				os.Setenv(containersConfEnv, oldEnv)
 			} else {
-				os.Unsetenv("CONTAINERS_CONF")
+				os.Unsetenv(containersConfEnv)
 			}
 
 			// Reload from new configuration file
@@ -823,48 +764,28 @@ env=["foo=bar"]`
 			err = os.WriteFile(testFile, []byte(content), os.ModePerm)
 			defer os.Remove(testFile)
 			gomega.Expect(err).To(gomega.BeNil())
-			oldEnv, set = os.LookupEnv("CONTAINERS_CONF")
-			os.Setenv("CONTAINERS_CONF", testFile)
+			oldEnv, set = os.LookupEnv(containersConfEnv)
+			os.Setenv(containersConfEnv, testFile)
 			_, err = Reload()
 			gomega.Expect(err).To(gomega.BeNil())
 			newCfg, err := Default()
 			gomega.Expect(err).To(gomega.BeNil())
 			if set {
-				os.Setenv("CONTAINERS_CONF", oldEnv)
+				os.Setenv(containersConfEnv, oldEnv)
 			} else {
-				os.Unsetenv("CONTAINERS_CONF")
+				os.Unsetenv(containersConfEnv)
 			}
 
-			expectOldEnv := []string{"PATH=/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin", "TERM=xterm"}
+			expectOldEnv := []string{"PATH=/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin"}
 			expectNewEnv := []string{"foo=bar"}
-			gomega.Expect(cfg.Containers.Env).To(gomega.Equal(expectOldEnv))
-			gomega.Expect(newCfg.Containers.Env).To(gomega.Equal(expectNewEnv))
+			gomega.Expect(cfg.Containers.Env.Get()).To(gomega.Equal(expectOldEnv))
+			gomega.Expect(newCfg.Containers.Env.Get()).To(gomega.Equal(expectNewEnv))
 			// Reload change back to default global configuration
 			_, err = Reload()
 			gomega.Expect(err).To(gomega.BeNil())
 		})
 	})
 
-	It("write default config should be empty", func() {
-		defer os.Unsetenv("CONTAINERS_CONF")
-		os.Setenv("CONTAINERS_CONF", "/dev/null")
-		conf, err := ReadCustomConfig()
-		gomega.Expect(err).ToNot(gomega.HaveOccurred())
-
-		f, err := os.CreateTemp("", "container-common-test")
-		gomega.Expect(err).ToNot(gomega.HaveOccurred())
-		defer f.Close()
-		defer os.Remove(f.Name())
-		os.Setenv("CONTAINERS_CONF", f.Name())
-		err = conf.Write()
-		gomega.Expect(err).ToNot(gomega.HaveOccurred())
-		b, err := os.ReadFile(f.Name())
-		gomega.Expect(err).ToNot(gomega.HaveOccurred())
-		// config should only contain empty stanzas
-		gomega.Expect(string(b)).To(gomega.
-			Equal("[containers]\n\n[engine]\n\n[machine]\n\n[network]\n\n[secrets]\n\n[configmaps]\n"))
-	})
-
 	It("validate ImageVolumeMode", func() {
 		for _, mode := range append(validImageVolumeModes, "") {
 			err := ValidateImageVolumeMode(mode)
@@ -873,4 +794,21 @@ env=["foo=bar"]`
 		err := ValidateImageVolumeMode("bogus")
 		gomega.Expect(err).To(gomega.HaveOccurred())
 	})
+
+	It("CONTAINERS_CONF_OVERRIDE", func() {
+		os.Setenv("CONTAINERS_CONF_OVERRIDE", "testdata/containers_override.conf")
+		defer os.Unsetenv("CONTAINERS_CONF_OVERRIDE")
+		config, err := NewConfig("")
+		gomega.Expect(err).ToNot(gomega.HaveOccurred())
+		gomega.Expect(config.Containers.ApparmorProfile).To(gomega.Equal("overridden-default"))
+
+		// Make sure that _OVERRIDE is loaded even when CONTAINERS_CONF is set.
+		os.Setenv(containersConfEnv, "testdata/containers_default.conf")
+		defer os.Unsetenv(containersConfEnv)
+		config, err = NewConfig("")
+		gomega.Expect(err).ToNot(gomega.HaveOccurred())
+		gomega.Expect(config.Containers.ApparmorProfile).To(gomega.Equal("overridden-default"))
+		gomega.Expect(config.Containers.BaseHostsFile).To(gomega.Equal("/etc/hosts2"))
+		gomega.Expect(config.Containers.EnableLabeledUsers).To(gomega.BeTrue())
+	})
 })
